@@ -120,6 +120,7 @@ from scalyr_agent.platform_controller import (
 )
 from scalyr_agent.platform_controller import AgentNotRunning
 from scalyr_agent.build_info import get_build_revision
+from scalyr_agent.metrics.base import clear_internal_cache
 from scalyr_agent import config_main
 from scalyr_agent import compat
 import scalyr_agent.monitors_manager
@@ -325,6 +326,8 @@ class ScalyrAgent(object):
 
         self.__config_file_path = config_file_path
 
+        no_check_remote = command_options.no_check_remote
+
         try:
             log_warnings = command not in ["status", "stop"]
             self.__config = self.__read_and_verify_config(
@@ -370,9 +373,6 @@ class ScalyrAgent(object):
             command_options.no_change_user,
         )
 
-        if command_options.no_check_remote is not None:
-            no_check_remote = True
-
         # noinspection PyBroadException
         try:
             # Execute the command.
@@ -1503,6 +1503,9 @@ class ScalyrAgent(object):
                         )
                     )
 
+                    # Clear metrics functions related cache
+                    clear_internal_cache()
+
                 # Log the stats one more time before we terminate.
                 self.__log_overall_stats(
                     self.__calculate_overall_stats(base_overall_stats)
@@ -2148,8 +2151,34 @@ if __name__ == "__main__":
     )
     command_parser.add_argument("command", choices=commands)
 
+    # Parse the main command by command parser.
+    raw_args = command_parser.parse_known_args()
+
+    # If argument parser ends with error, it does not exit automatically and just returns None.
+    if raw_args:
+        args, other_argv = raw_args
+        if args.command == "service":
+            # Windows specific command that tell to start Agent's Windows service.
+            from scalyr_agent import platform_windows
+
+            # Create fully valid command line args so the Windows service could handle it properly.
+            argv = [sys.argv[0]]
+            argv.extend(other_argv)
+            platform_windows.parse_options(argv)
+            exit(0)
+
+        if args.command == "config":
+            # Add the config option. So we can configure the agent from the same executable.
+            # It has to save us from building multiple frozen binaries, when packaging.
+            config_main.parse_config_options(other_argv)
+            exit(0)
+
     # Also create parser for the agent command line.
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(
+        usage="scalyr-agent-2 [options] ({})".format("|".join(commands)),
+    )
+
+    parser.add_argument("command", help="Agent command to run", choices=commands)
 
     parser.add_argument(
         "-c",
@@ -2213,31 +2242,7 @@ if __name__ == "__main__":
     )
     my_controller.add_options(parser)
 
-    # Parse the main command by command parser.
-    command_args, other_argv = command_parser.parse_known_args()
-
-    # If command is not specified, then we just print usage of the this script.
-    if command_args.command is None:
-        parser.print_help(file=sys.stderr)
-        exit(1)
-
-    if command_args.command == "service":
-        # Windows specific command that tell to start Agent's Windows service.
-        from scalyr_agent import platform_windows
-
-        # Create fully valid command line args so the Windows service could handle it properly.
-        argv = [sys.argv[0]]
-        argv.extend(other_argv)
-        platform_windows.parse_options(argv)
-        exit(0)
-
-    if command_args.command == "config":
-        # Add the config option. So we can configure the agent from the same executable.
-        # It has to save us from building multiple frozen binaries, when packaging.
-        config_main.parse_config_options(other_argv)
-        exit(0)
-
-    options = parser.parse_args(args=other_argv)
+    options = parser.parse_args()
 
     my_controller.consume_options(options)
 
@@ -2249,7 +2254,7 @@ if __name__ == "__main__":
     main_rc = 1
     try:
         main_rc = ScalyrAgent(my_controller).main(
-            options.config_filename, command_args.command, options
+            options.config_filename, options.command, options
         )
     except Exception as mainExcept:
         print(six.text_type(mainExcept), file=sys.stderr)
