@@ -38,23 +38,30 @@ import tempfile
 import traceback
 import errno
 import itertools
+import argparse
 from io import open
 
-from optparse import OptionParser
-
 # TODO: The following two imports have been modified to facilitate Windows platforms
 if not sys.platform.startswith("win"):
     from pwd import getpwnam
 
 # pylint: disable=import-error
-from __scalyr__ import (
-    scalyr_init,
-    get_install_root,
-    TARBALL_INSTALL,
-    MSI_INSTALL,
-    SCALYR_VERSION,
-    PACKAGE_INSTALL,
-)
+try:
+    from scalyr_agent.__scalyr__ import (
+        scalyr_init,
+        get_install_root,
+        TARBALL_INSTALL,
+        SCALYR_VERSION,
+        PACKAGE_INSTALL,
+    )
+except ImportError:
+    from __scalyr__ import (
+        scalyr_init,
+        get_install_root,
+        TARBALL_INSTALL,
+        SCALYR_VERSION,
+        PACKAGE_INSTALL,
+    )
 
 # pylint: enable=import-error
 
@@ -644,7 +651,10 @@ def upgrade_windows_install(
         my_default_paths = platform_controller.default_paths
 
         # Ensure agent was installed via MSI
-        if MSI_INSTALL != platform_controller.install_type:
+        if (
+            platform_controller.install_type != PACKAGE_INSTALL
+            or platform.system() != "Windows"
+        ):
             raise UpgradeFailure(
                 "The current agent was not installed via MSI, so you may not use the upgrade windows "
                 "command."
@@ -752,7 +762,7 @@ def upgrade_windows_install(
                 else:
                     print("Executing upgrade.  It will finish in the background.")
 
-                # Because this file, config_main.py, is part of the currently installed Scalyr Agent package, we have
+                # Because this file, agent_config.py, is part of the currently installed Scalyr Agent package, we have
                 # to finish our use of it before the upgrade can proceed.  So, we just fork off the msiexec process
                 # in detached mode and terminate this program.  This means we cannot report any errors that happen
                 # here, but I don't see a way around this for now.
@@ -1231,21 +1241,21 @@ def set_python_version(version):
     # use on the 'python' command and rely on the python version which it mapped on.
     if version in DEFAULT:
         agent_main_filename = "agent_main.py"
-        config_main_filename = "config_main.py"
+        # config_main_filename = "agent_config.py"
     # python 'python2
     elif version == PYTHON2:
         agent_main_filename = "agent_main_py2.py"
-        config_main_filename = "config_main_py2.py"
+        # config_main_filename = "config_main_py2.py"
     # python 'python3
     else:
         agent_main_filename = "agent_main_py3.py"
-        config_main_filename = "config_main_py3.py"
+        # config_main_filename = "config_main_py3.py"
 
     agent_main_source = os.path.join(source_path, agent_main_filename)
-    config_main_source = os.path.join(source_path, config_main_filename)
+    # config_main_source = os.path.join(source_path, config_main_filename)
 
     scalyr_agent_2_target = os.path.join(binary_path, "scalyr-agent-2")
-    scalyr_agent_2_config_target = os.path.join(binary_path, "scalyr-agent-2-config")
+    # scalyr_agent_2_config_target = os.path.join(binary_path, "scalyr-agent-2-config")
 
     def make_symlink(source, target):
         try:
@@ -1257,7 +1267,7 @@ def set_python_version(version):
 
     # recreate symlinks to agent main and config_main.
     make_symlink(agent_main_source, scalyr_agent_2_target)
-    make_symlink(config_main_source, scalyr_agent_2_config_target)
+    # make_symlink(config_main_source, scalyr_agent_2_config_target)
 
     print("Switched agent to {0}".format(version))
     print(
@@ -1388,17 +1398,17 @@ def configure_agent_service_for_systemd_management():
     )
 
 
-if __name__ == "__main__":
-    parser = OptionParser(usage="Usage: scalyr-agent-2-config [options]")
-    parser.add_option(
+def parse_config_options(argv):
+
+    parser = argparse.ArgumentParser()
+    parser.add_argument(
         "-c",
         "--config-file",
         dest="config_filename",
         help="Read configuration from FILE",
         metavar="FILE",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--set-key-from-stdin",
         action="store_true",
         dest="set_key_from_stdin",
@@ -1406,23 +1416,20 @@ if __name__ == "__main__":
         help="Update the configuration file with a new API key read from standard input.  "
         "The API key is used to authenticate requests to the Scalyr servers for the account.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--set-key",
         dest="api_key",
         help="Update the configuration file with the new API key."
         "The API key is used to authenticate requests to the Scalyr servers for the account.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--set-scalyr-server",
         dest="scalyr_server",
         help="Updates the configuration to send all log uploads to the specified server.  This will "
         "create a configuration file fragment `scalyr_server.json` in the config directory.  It "
         "will overwrite any existing file at that path.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--set-server-host",
         dest="server_host",
         help="Adds a new configuration file in the ``agent.d`` directory to set the serverHost "
@@ -1431,14 +1438,12 @@ if __name__ == "__main__":
         "must be sure the ``agent.json`` file nor any file in ``agent.d`` sets a value for "
         "``serverHost`` otherwise this might not work.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--set-user",
         dest="executing_user",
         help="Update which user account is used to run the agent.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--upgrade-tarball",
         dest="upgrade_tarball",
         help="Upgrade the agent to the new version contained in the specified tarball file."
@@ -1452,8 +1457,7 @@ if __name__ == "__main__":
         "which will be copied over to the new installation).  If you may have modified other files "
         "then use the --preserve-old-install option to prevent it from being deleted.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--preserve-old-install",
         action="store_true",
         dest="preserve_old_install",
@@ -1461,8 +1465,7 @@ if __name__ == "__main__":
         help="When performing a tarball upgrade, move the old install to a temporary directory "
         "instead of deleting it.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--import-config",
         dest="import_config",
         help="Extracts the agent configuration files from the provided gzipped tarball, overwriting the"
@@ -1472,16 +1475,14 @@ if __name__ == "__main__":
         "extracted files are reset to be the same owner/group of the current configuration file to "
         "avoid permission problems.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--export-config",
         dest="export_config",
         help="Creates a new gzipped tarball using the current agent configuration files stored in the "
         "`agent.json` file and the `agent.d` directory.  Pass `-` to write the tarball to stdout. "
         "Note, this only copies files that end in `.json`.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--docker-create-custom-dockerfile",
         dest="create_custom_dockerfile",
         help="Creates a gzipped tarball that will extract to a Dockerfile that will build a custom "
@@ -1491,8 +1492,7 @@ if __name__ == "__main__":
         "containers.  The option value should either be a path to write the tarball or `-` to "
         "write it to stdout.",
     )
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--k8s-create-custom-dockerfile",
         dest="create_custom_k8s_dockerfile",
         help="Creates a gzipped tarball that will extract to a Dockerfile that will build a custom "
@@ -1504,16 +1504,14 @@ if __name__ == "__main__":
         "write it to stdout.",
     )
 
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--set-python",
         dest="set_python",
         choices=DEFAULT + [PYTHON2, PYTHON3],
         help="Switch current python interpreter. Can be selected from python2 and python3.",
     )
 
-    parser.add_option(
-        "",
+    parser.add_argument(
         "--report-python-version",
         action="store_true",
         dest="report_python_version",
@@ -1523,24 +1521,21 @@ if __name__ == "__main__":
 
     # TODO: These options are only available on Windows platforms
     if sys.platform.startswith("win"):
-        parser.add_option(
-            "",
+        parser.add_argument(
             "--upgrade-windows",
             dest="upgrade_windows",
             action="store_true",
             default=False,
             help="Upgrade the agent if a new version is available",
         )
-        parser.add_option(
-            "",
+        parser.add_argument(
             "--release-track",
             dest="release_track",
             default="stable",
             help="The release track to use when upgrading using --upgrade-windows.  This defaults to "
             '"stable" and is what consumers should use.',
         )
-        parser.add_option(
-            "",
+        parser.add_argument(
             "--upgrade-without-ui",
             dest="upgrade_windows_no_ui",
             action="store_true",
@@ -1550,16 +1545,14 @@ if __name__ == "__main__":
         )
         # TODO: Once other packages (rpm, debian) include the 'templates' directory, we can make this available
         # beyond just Windows.
-        parser.add_option(
-            "",
+        parser.add_argument(
             "--init-config",
             dest="init_config",
             action="store_true",
             default=False,
             help="Create an initial copy of the configuration file in the appropriate location.",
         )
-        parser.add_option(
-            "",
+        parser.add_argument(
             "--no-error-if-config-exists",
             dest="init_config_ignore_exists",
             action="store_true",
@@ -1569,8 +1562,7 @@ if __name__ == "__main__":
         # These are a weird options we use to start the agent after the Windows install process finishes if there
         # was an agent running before.  If there was an agent, the write a special file
         # called the conditional restart marker.  So, if it exists, then we should start the agent.
-        parser.add_option(
-            "",
+        parser.add_argument(
             "--mark-conditional-restart",
             dest="mark_conditional_restart",
             action="store_true",
@@ -1578,8 +1570,7 @@ if __name__ == "__main__":
             help="Creates the marker file to restart the agent next time --conditional-restart is "
             "specified if the agent is currently running.",
         )
-        parser.add_option(
-            "",
+        parser.add_argument(
             "--conditional-restart",
             dest="conditional_restart",
             action="store_true",
@@ -1592,8 +1583,7 @@ if __name__ == "__main__":
         # to create initial config by the install which means we can't correctly set those
         # permissions inside the .wxs wix spec file.
         # Right now it can only be used with "--init-config" flag on Windows.
-        parser.add_option(
-            "",
+        parser.add_argument(
             "--fix-config-permissions",
             dest="fix_config_permissions",
             action="store_true",
@@ -1606,8 +1596,7 @@ if __name__ == "__main__":
 
     # Add Linux specific options
     if sys.platform.startswith("linux"):
-        parser.add_option(
-            "",
+        parser.add_argument(
             "--systemd-managed",
             dest="systemd_managed",
             action="store_true",
@@ -1615,18 +1604,13 @@ if __name__ == "__main__":
             help="Configure the agent to be managed by systemd instead of init.d which is a default",
         )
 
-    (options, args) = parser.parse_args()
-    if len(args) > 1:
-        print("Could not parse commandline arguments.", file=sys.stderr)
-        parser.print_help(sys.stderr)
-        sys.exit(1)
-
     controller = PlatformController.new_platform()
     default_paths = controller.default_paths
 
     # NOTE: This piece of code should be at the top before we parse the config since the script
     # can run as part of postinstall step when the config is not present yet. And in general, that
     # operation should be standalone without any reliance on the agent config.
+    options = parser.parse_args(args=argv)
     if options.set_python is not None:
         set_python_version(options.set_python)
         print("Agent switched to {0}.".format(options.set_python))
@@ -1799,3 +1783,7 @@ if __name__ == "__main__":
         )
 
     sys.exit(0)
+
+
+if __name__ == "__main__":
+    parse_config_options(sys.argv)
